#include "forecast.h"
#include "weather.h"
#include "net.h"
#include "../utils.h"
#include "../file.h"
#include "../ime_dialog.h"
#include "../theme.h"
#include "../language.h"
#include "../main.h"


static char patch[MAX_PATH_LENGTH];

void initWeatherData()
{
	char buffer[1024], buffer2[2];
	
	if (!(checkFolderExist("ux0:/data/VITAforecast")))
	{
		SceUID dir;
		
		dir = sceIoDopen("ux0:/data/VITAforecast");
		sceIoMkdir("ux0:/data/VITAforecast", 0777);
		sceIoDclose(dir);
	}
	
	if (checkFileExist("ux0:data/VITAforecast/zipCode.txt"))
		ReadFile("ux0:data/VITAforecast/zipCode.txt", zip, 12);
	else
	{
		strcpy(zip, "31100,fr");
		WriteFile("ux0:data/VITAforecast/zipCode.txt", zip, 12);
		downloadWeatherData();
	}
	
	if (checkFileExist("ux0:data/VITAforecast/weather.txt"))
		ReadFile("ux0:data/VITAforecast/weather.txt", buffer, 1024);
	
	if (checkFileExist("ux0:data/VITAforecast/metricUnits.txt"))
	{
		ReadFile("ux0:data/VITAforecast/metricUnits.txt", buffer2, 2);
		sscanf(buffer2, "%d", &CorF);
	}
	else
	{
		if (strcmp((getWeatherContent(buffer, "country\":\"", "\",\"sunrise")), "US") == 0)
		{
			sprintf(buffer2, "0");
			WriteFile("ux0:data/VITAforecast/metricUnits.txt", buffer2, 2);
			sscanf(buffer2, "%d", &CorF);
		}
		else
		{
			sprintf(buffer2, "1");
			WriteFile("ux0:data/VITAforecast/metricUnits.txt", buffer2, 2);
			sscanf(buffer2, "%d", &CorF);
		}
	}
}

char * urlConcat(char url[], char zipCode[], char apiKey[])
{
	static char str[100];
	strcpy(str, url);
	strcat(str, zipCode); 
	strcat(str, apiKey); 
	
	return str;
}

int downloadWeatherData()
{
	char url[100];
	strcpy(url, urlConcat("http://api.openweathermap.org/data/2.5/weather?zip=", zip, "&appid=1fbac727d98de6617dc510dd3b4aac4b"));
	while(1)
	{
		
		download(url, "ux0:data/VITAforecast/weather.txt");
		return 1;
	}
}

char * getWeatherContent(char * string, char * str1, char * str2) //Basically finds a string between two othe strings :D
{
	static char str[1024];
	char *begin, *end, *crawl;
	int i = 0;
	
	begin = strstr(string, str1);
	begin+=strlen(str1);
	
	end = strstr(string, str2);	
	
	for(crawl = begin; crawl < end; crawl++)
		str[i++] = *crawl;
		
	str[i] = '\0'; //add '\0' to the array so that it's a string!
	
	return str;
}

double strToDouble(char * str)
{
	double val = 0.0;
	
	sscanf(str, "%lf", &val);
	
	return val;
}

double kelvinToC(double temp)
{
	double c = (temp - 273);
	
	return c;
}	

double cToF(double temp)
{
	double c = (temp - 273);
	double f = (c * (1.8)) + 32.0;
	
	return f;
}	

double mToKm(double m)
{
	double km = (m * 3.6);
	return km;
}

double mToMph(double m)
{
	double mph = (m * 2.23694);
	return mph;
}

bool isDay()//basic assumption really
{
	SceDateTime time;
	sceRtcGetCurrentClockLocalTime(&time);
		
	if ((time.hour >= 6) && (time.hour <= 18))
		return true; //Day
	else 
		return false; //Night
}

void displayWeatherContent()
{
	char buffer[1024];
	
	double c = 0.0, cMin = 0.0, cMax = 0.0, windSpeed = 0.0;
	
	ReadFile("ux0:data/VITAforecast/weather.txt", buffer, 1024);
	
	float slide_x = ForeCast_menu.cur_pos - 480;
	
	if((getFileSize("ux0:data/VITAforecast/weather.txt")) <= 0)
	{
		pgf_draw_text(30+slide_x, 80, FORECAST_ERROR_COLOR, 1, "Error receiving data. The data could not be downloaded.");
		pgf_draw_text(30+slide_x, 120, FORECAST_ERROR_COLOR, 1, "Please ensure that you have a working net connection.");
	}
	else if (strcmp(buffer, "{\"cod\":\"500\",\"message\":\"Internal error: 500000\"}") == 0)
	{
		pgf_draw_text(30+slide_x, 80, FORECAST_ERROR_COLOR, 1, "Error parsing data. Zip code cannot be validated.");
		pgf_draw_text(30+slide_x, 120, FORECAST_ERROR_COLOR, 1, "Press square and renter zip and country code.");
	}
	else
	{
		displayWeatherIcon(getWeatherContent(buffer, "main\":\"", "\",\"des"), isDay());
	
		snprintf(patch, MAX_PATH_LENGTH, "%s", getWeatherContent(buffer, "name\":\"", "\",\"cod"));
		pgf_draw_text(30+slide_x, 70, FORECAST_NAME_COLOR, 1.5, patch);
	
		c = strToDouble(getWeatherContent(buffer, "temp\":", ",\"pressure"));
		cMin = strToDouble(getWeatherContent(buffer, "temp_min\":", ",\"temp_max"));
		cMax = strToDouble(getWeatherContent(buffer, "temp_max\":", "},\"visibility"));
		windSpeed = strToDouble(getWeatherContent(buffer, "speed\":", ",\"deg"));
	
		if (CorF == 0)
		{
			snprintf(patch, MAX_PATH_LENGTH, "%.0lf°F", cToF(c));
			pgf_draw_text(90+slide_x, 140, FORECAST_TEMP_COLOR, 1, patch);
			snprintf(patch, MAX_PATH_LENGTH, "Min: %.0lf°F", cToF(cMin));
			pgf_draw_text(70+slide_x, 190, FORECAST_TEMP_MIN_COLOR, 1, patch);
			snprintf(patch, MAX_PATH_LENGTH, "Max: %.0lf°F", cToF(cMax));
			pgf_draw_text(70+slide_x, 210, FORECAST_TEMP_MAX_COLOR, 1, patch);
			snprintf(patch, MAX_PATH_LENGTH, "%s: %.02f mph", language_container[FORECAST_WIND], mToMph(windSpeed));
			pgf_draw_text(30+slide_x, 430, FORECAST_WIND_COLOR, 1, patch);
			
			if (pressed_buttons & SCE_CTRL_SELECT)
			{
				CorF = 1;
				WriteFile("ux0:data/VITAforecast/metricUnits.txt", "1", 2);
			}
		}
		else
		{
			snprintf(patch, MAX_PATH_LENGTH, "%.0lf°C", kelvinToC(c));
			pgf_draw_text(90+slide_x, 140, FORECAST_TEMP_COLOR, 1, patch);
			snprintf(patch, MAX_PATH_LENGTH, "Min: %.0lf°C", kelvinToC(cMin));
			pgf_draw_text(70+slide_x, 190, FORECAST_TEMP_MIN_COLOR, 1, patch);
			snprintf(patch, MAX_PATH_LENGTH, "Max: %.0lf°C", kelvinToC(cMax));
			pgf_draw_text(70+slide_x, 210, FORECAST_TEMP_MAX_COLOR, 1, patch);
			snprintf(patch, MAX_PATH_LENGTH, "%s: %.02f km/h", language_container[FORECAST_WIND], mToKm(windSpeed));
			pgf_draw_text(30+slide_x, 430, FORECAST_WIND_COLOR, 1, patch); 
			
			if (pressed_buttons & SCE_CTRL_SELECT)
			{
				CorF = 0;
				WriteFile("ux0:data/VITAforecast/metricUnits.txt", "0", 2);
			}
		}
		
		snprintf(patch, MAX_PATH_LENGTH, "%s\n", getWeatherContent(buffer, "description\":\"", "\",\"icon"));
		float x = vita2d_pgf_text_width(font, FONT_SIZE, patch);
		pgf_draw_text(ALIGN_CENTER(480, x)+slide_x, 305, FORECAST_DESC_COLOR, 1, patch);
		snprintf(patch, MAX_PATH_LENGTH, "%s: %s%%", language_container[FORECAST_HUMID], getWeatherContent(buffer, "humidity\":", ",\"temp_"));
		pgf_draw_text(30+slide_x, 350, FORECAST_HUMID_COLOR, 1, patch);
		snprintf(patch, MAX_PATH_LENGTH, "%s: %s%%", language_container[FORECAST_CLOUD], getWeatherContent(buffer, "clouds\":{\"all\":", "},\"dt"));
		pgf_draw_text(30+slide_x, 390, FORECAST_CLOUD_COLOR, 1, patch);
		snprintf(patch, MAX_PATH_LENGTH, "%s: %s hPa", language_container[FORECAST_ATMOS], getWeatherContent(buffer, "pressure\":", ",\"humidity"));
		pgf_draw_text(30+slide_x, 470, FORECAST_ATMOS_COLOR, 1, patch);
	}
}

void displayWeatherIcon(char * desc, bool day) //Uses main description
{
	float slide_x = ForeCast_menu.cur_pos - 480;
	
	if (day == true)
	{
		if (strcmp(desc, "Clear") == 0)
			vita2d_draw_texture(dClearSky, 310+slide_x, 120);
		else if (strcmp(desc, "Clouds") == 0)
			vita2d_draw_texture(dClouds, 310+slide_x, 120);
		else if ((strcmp(desc, "Rain") == 0) || (strcmp(desc, "Drizzle") == 0))
			vita2d_draw_texture(dRain, 310+slide_x, 120);
		else if ((strcmp(desc, "Thunderstorm") == 0) || (strcmp(desc, "Extreme") == 0))
			vita2d_draw_texture(dThunderstorm, 310+slide_x, 120);
		else if (strcmp(desc, "Snow") == 0)
			vita2d_draw_texture(dSnow, 310+slide_x, 120);
		else if (strcmp(desc, "Atmosphere") == 0)
			vita2d_draw_texture(mist, 310+slide_x, 120);
	}
	else
	{
		if (strcmp(desc, "Clear") == 0)
			vita2d_draw_texture(nClearSky, 310+slide_x, 120);
		else if (strcmp(desc, "Clouds") == 0)
			vita2d_draw_texture(nClouds, 310+slide_x, 120);
		else if ((strcmp(desc, "Rain") == 0) || (strcmp(desc, "Drizzle") == 0))
			vita2d_draw_texture(nRain, 310+slide_x, 120);
		else if ((strcmp(desc, "Thunderstorm") == 0) || (strcmp(desc, "Extreme") == 0))
			vita2d_draw_texture(nThunderstorm, 310+slide_x, 120);
		else if (strcmp(desc, "Snow") == 0)
			vita2d_draw_texture(nSnow, 310+slide_x, 120);
		else if (strcmp(desc, "Atmosphere") == 0)
			vita2d_draw_texture(mist, 310+slide_x, 120);
	}
}